//********************************************************************
UNIT GUIBase;
//********************************************************************
INTERFACE

USES SysUtils, Contnrs, GraphicSupport;
//********************************************************************
TYPE
   TGUIClassName = String;
   TGUIIdentity = String;
   TGUIColorName = String;
   TGUIColorData = Record ColName: String; RGBAColor: TRGBAColor; End;
   TGUIFontName = String;
   TGUIFontData = Record
      FontName: TGUIFontName;
      FontSize,FontStyle: Integer;
   End;
   TGUIPosition = Longint;
   TGUIPositionRect = Record
      Left,Top,Right,Bottom: TGUIPosition;
   End;
   TGUITextureName = String;
   TGUITextureData = Record
      TextureName: TGUITextureName;
      TextureTranspColor: TRGBAColor;
   End;
   TGUIVisible = Boolean;
   TGUIThemesIndex = LongInt;
//********************************************************************
   TGUIDataProvider = Class(TObject)
   // Open/Close
      Function OpenSource(ASourceName: String): Boolean; Virtual; Abstract;
      Procedure CloseSource; Virtual; Abstract;

   // Data loaded by Themes
      Function LoadCount: Integer; Virtual; Abstract;
      Procedure SaveCount(ACount: Integer); Virtual; Abstract;
      Function LoadColorData: TGUIColorData; Virtual; Abstract;
      Procedure SaveColorData(AColorData: TGUIColorData); Virtual; Abstract;
      Function LoadTextureData: TGUITextureData; Virtual; Abstract;
      Procedure SaveTextureData(ATextureData: TGUITextureData); Virtual; Abstract;
      Function LoadFontData: TGUIFontData; Virtual; Abstract;
      Procedure SaveFontData(AFontData: TGUIFontData); Virtual; Abstract;

   // Data loaded by TGUIItems + Descendents
      Function LoadClassName: TGUIClassName; Virtual; Abstract;
      Procedure SaveClassName(AClassName: TGUIClassName); Virtual; Abstract;
      Function LoadIdentity: TGUIIdentity; Virtual; Abstract;
      Procedure SaveIdentity(AIdentity: TGUIIdentity); Virtual; Abstract;
      Function LoadPositionRect: TGUIPositionRect; Virtual; Abstract;
      Procedure SavePositionRect(APositionRect: TGUIPositionRect); Virtual; Abstract;
      Function LoadVisible: TGUIVisible; Virtual; Abstract;
      Procedure SaveVisible(AVisible: TGUIVisible); Virtual; Abstract;
      Function LoadColorName: TGUIColorName; Virtual; Abstract;
      Procedure SaveColorName(AColorName: TGUIColorName); Virtual; Abstract;
      Function LoadTextureName: TGUITextureName; Virtual; Abstract;
      Procedure SaveTextureName(ATextureName: TGUITextureName); Virtual; Abstract;
      Function LoadFontName: TGUIFontName; Virtual; Abstract;
      Procedure SaveFontName(AFontName: TGUIFontName); Virtual; Abstract;
   End;
//********************************************************************
{$M+}
   TGUIColor = Class(TObject)
   Private
      FName: String;
      FBaseColor: TRGBAColor;
      FShownColor: TRGBAColor;

       Function GetRGBAColor: TRGBAColor;
      Procedure SetRGBAColor(ARGBAColor: TRGBAColor);
      Procedure SetLuminance(ALuminance: Single);
      Procedure SetRed(ARed: Single);
      Procedure SetBlue(ABlue: Single);
      Procedure SetGreen(AGreen: Single);
      Procedure SetAlpha(AAlpha: Single);
   Public
      Property Name: String Read FName Write FName;
      Property Color: TRGBAColor Read GetRGBAColor Write SetRGBAColor;
      Property Luminance: Single Write SetLuminance;
      Property Red: Single Write SetRed;
      Property Blue: Single Write SetBlue;
      Property Green: Single Write SetGreen;
      Property Alpha: Single Write SetAlpha;

      Procedure Load(ADataProvider: TGUIDataProvider);
      Procedure Save(ADataProvider: TGUIDataProvider);
      Procedure Reset;
   End;
{$M-}
//********************************************************************
   TGUIThemes = Class(TObject)
   Private
      FColor: TGUIColor;
      FTexture: TGRS_Texture;
      FFont: TGRS_TTFont;

      FLineWidth: Integer;

      FColors: TObjectList;
      FFonts: TObjectList;
      FTextures: TObjectList;
      FDataProvider: TGUIDataProvider;

       Function GetColor: TGUIColor;
      Procedure SetColor(AColor: TGUIColor);
       Function GetLineWidth: Integer;
      Procedure SetLineWidth(ALineWidth: Integer);
   Public
      Property Color: TGUIColor
         Read GetColor Write SetColor;
      Property Font: TGRS_TTFont Read FFont;
      Property Texture: TGRS_Texture Read FTexture;
      Property LineWidth: Integer
         Read GetLineWidth Write SetLineWidth;
      Property DataProvider: TGUIDataProvider
         Write FDataProvider;

      Constructor Create; Reintroduce;
      Destructor Destroy; Override;
        Function Initialize: Boolean;

      // Administrate colors
      Function ActivateColor(AColorIndex: TGUIThemesIndex): Boolean;
      Function AddColor(AColorName: String;
                   ARGBAColor: TRGBAColor): TGUIThemesIndex;
      Function ColorIndex(AColorName: TGUIColorName): TGUIThemesIndex;
      Procedure LoadColors;
      Procedure SaveColors;

      // Administrate textures
      Function ActivateTexture(ATextureIndex: TGUIThemesIndex): Boolean;
      Function AddTexture(ATextureName: String;
             ATransparentColor: TRGBAColor): TGUIThemesIndex;
      Function TextureIndex(ATextureName: TGUITextureName): TGUIThemesIndex;
      Procedure LoadTextures;
      Procedure SaveTextures;

      // Administrate fonts
      Function ActivateFont(AFontIndex: TGUIThemesIndex): Boolean;
      Function AddFont(AFontName: String;
                 ASize,AStyle: Integer): TGUIThemesIndex;
      Function FontIndex(AFontName: TGUIFontName): TGUIThemesIndex;
      Procedure LoadFonts;
      Procedure SaveFonts;

      // DRAWING PROCEDURES WITHOUT TEXTURE --------------------------
      // Parameters = ALeft,ATop,ARight,ABottom
      Procedure Ellipse(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);
      Procedure Line(FromX,FromY,ToX,ToY: Integer);
      Procedure Rectangle(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);
      Procedure RoundRect(ALeft,ATop,ARight,ABottom: Integer;
                                       Fill: Boolean);

      // Parameters = Array Of TPoint2D
      Procedure Graph(APoint: Array Of TPoint2D);
      Procedure Polygon(APoint: Array Of TPoint2D; Fill: Boolean);

      // DRAWING PROCEDURES WITH TEXTURE -----------------------------
      // Parameters = ALeft,ATop,ARight,ABottom
      Procedure TexRectangle(ALeft,ATop,ARight,ABottom: Integer;
                                 WithTexture: Boolean);
   End;
//********************************************************************
   TMouseImage = (miArrow,miCaret);
//********************************************************************
   TMouse = Class(TObject)
   Private
      FMouseImage: TMouseImage;
   Public
      Constructor Create; Reintroduce;
      Destructor Destroy; Override;
      Procedure Draw;
       Function GetMouseImage: TMouseImage;
      Procedure SetMouseImage(AMouseImage: TMouseImage);
   End;
//********************************************************************
VAR MainDir, FontDir, TextureDir: String;
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
//   T G U I C O L O R
//********************************************************************
// No comment
Function TGUIColor.GetRGBAColor: TRGBAColor;
Begin
   Result:= FShownColor;
End;
//********************************************************************
// Sets bothe the base color and the shown color
Procedure TGUIColor.SetRGBAColor(ARGBAColor: TRGBAColor);
Begin
   FBaseColor:= ARGBAColor;
   FShownColor:= FBaseColor;
End;
//********************************************************************
// No comment
Procedure TGUIColor.Load(ADataProvider: TGUIDataProvider);
Var ColorData: TGUIColorData;
Begin
   ColorData:= ADataProvider.LoadColorData;
   With ColorData Do Begin
      Name:= ColName;
      FBaseColor:= RGBAColor;
   End;
End;
//********************************************************************
// No comment
Procedure TGUIColor.Save(ADataProvider: TGUIDataProvider);
Var ColorData: TGUIColorData;
Begin
   With ColorData Do Begin
      ColName:= Name;
      RGBAColor:= FBaseColor;
   End;
   ADataProvider.SaveColorData(ColorData);
End;
//********************************************************************
// Sets the shown color back
Procedure TGUIColor.Reset;
Begin
   FShownColor:= FBaseColor;
End;
//********************************************************************
// Set the luminance of the shown color
Procedure TGUIColor.SetLuminance(ALuminance: Single);
Begin
   If (ALuminance >= 0) And (ALuminance <= 1)
      Then With FShownColor Do Begin
           Red:= FBaseColor.Red * ALuminance;
         Green:= FBaseColor.Green * ALuminance;
          Blue:= FBaseColor.Blue * ALuminance;
      End;
End;
//********************************************************************
// No comment
Procedure TGUIColor.SetRed(ARed: Single);
Begin
   FShownColor.Red:= ARed;
End;
//********************************************************************
// No comment
Procedure TGUIColor.SetBlue(ABlue: Single);
Begin
   FShownColor.Blue:= ABlue;
End;
//********************************************************************
// No comment
Procedure TGUIColor.SetGreen(AGreen: Single);
Begin
   FShownColor.Green:= AGreen;
End;
//********************************************************************
// No comment
Procedure TGUIColor.SetAlpha(AAlpha: Single);
Begin
   FShownColor.Alpha:= AAlpha;
End;
//********************************************************************
//********************************************************************
//   T G U I T H E M E S
//********************************************************************
// No comment
Constructor TGUIThemes.Create;
Begin
   Inherited Create;

   FColors:= TObjectList.Create;
   FTextures:= TObjectList.Create;
   FFonts:= TObjectList.Create;
End;
//********************************************************************
// No comment
Destructor TGUIThemes.Destroy;
Begin
   FFonts.Free;
   FTextures.Free;
   FColors.Free;

   Inherited Destroy;
End;
//********************************************************************
// No comment
Function TGUIThemes.Initialize: Boolean;
Begin
   AddColor('SOLIDWHITE', SolidWhite);
   AddColor('TRANSPWHITE', TranspWhite);
   AddColor('SOLIDBLACK', SolidBlack);
   AddColor('SOLIDGREY', SolidGrey);

   AddColor('SOLIDRED', SolidRed);
   AddColor('SOLIDGREEN', SolidGreen);
   AddColor('SOLIDBLUE', SolidBlue);

   AddColor('SOLIDYELLOW', SolidYellow);
   AddColor('SOLIDCYAN', SolidCyan);
   AddColor('SOLIDMAGENTA', SolidMagenta);
   AddColor('SOLIDORANGE', SolidOrange);

   With FColors Do FColor:= List[0];

   AddTexture('Default64X64', SolidBlack);
   With FTextures Do FTexture:= List[0];

   AddFont('FreeMono', 100, TTF_STYLE_NORMAL);
   With FFonts Do FFont:= List[0];

   If (FColors.Count > 0) And
      (FTextures.Count > 0) And
      (FFonts.Count > 0)
      Then Result:= True
      Else Result:= False;
End;
//********************************************************************
// No comment
Function TGUIThemes.GetColor: TGUIColor;
Begin Result:= FColor; End;
//********************************************************************
// No comment
Procedure TGUIThemes.SetColor(AColor: TGUIColor);
Begin FColor:= AColor; End;
//********************************************************************
// No comment
Function TGUIThemes.GetLineWidth: Integer;
Begin Result:= FLineWidth; End;
//********************************************************************
// No comment
Procedure TGUIThemes.SetLineWidth(ALineWidth: Integer);
Begin FLineWidth:= ALineWidth; End;
//********************************************************************
//   T G U I T H E M E S   -   Administrate colors
//********************************************************************
Function TGUIThemes.ActivateColor(AColorIndex: TGUIThemesIndex): Boolean;
Begin
   Result:= False;
   With FColors Do
      If (Count > 0) And
         (AColorIndex >= 0) And
         (AColorIndex < Count)
         Then Begin
            FColor:= List[AColorIndex];
            Result:= True;
         End;
End;
//********************************************************************
// AddColor creates a TGUIColor with the passed color and adds it
// to the colorlist
Function TGUIThemes.AddColor(AColorName: String;
                     ARGBAColor: TRGBAColor): TGUIThemesIndex;
Var LocalColor: TGUIColor;
Begin
   LocalColor:= TGUIColor.Create;
   With LocalColor Do Begin
      Name:= AColorName;
      Color:= ARGBAColor;
   End;
   FColor:= LocalColor;
   FColors.Add(LocalColor);
   Result:= FColors.Count-1;
End;
//********************************************************************
// The names of the listed colors are supposed to be upper case
Function TGUIThemes.ColorIndex(AColorName: TGUIColorName): TGUIThemesIndex;
Var LocalColor: TGUIColor; I: Integer;
Begin
   Result:= -1;
   AColorName:= UpperCase(AColorName);
   I:= 0;
   With FColors Do If Count > 0
      Then While (I < Count) And (Result < 0) Do Begin
         LocalColor:= List[I];
         If LocalColor.Name <> AColorName
            Then Inc(I) //=================>
            Else Result:= I;
      End;
End;
//********************************************************************
Procedure TGUIThemes.LoadColors;
Var LocalColor: TGUIColor; ColorCount, I: Integer;
Begin
   ColorCount:= FDataProvider.LoadCount;
   For I:= 0 To ColorCount-1 Do Begin
      LocalColor:= TGUIColor.Create;
      LocalColor.Load(FDataProvider);
      FColors.Add(LocalColor);
   End;
End;
//********************************************************************
Procedure TGUIThemes.SaveColors;
Var LocalColor: TGUIColor; I: Integer;
Begin
   With FColors Do Begin
      FDataProvider.SaveCount(Count);
      For I:= 0 To Count-1 Do Begin
         LocalColor:= List[I];
         LocalColor.Save(FDataProvider);
      End;
   End;
End;
//********************************************************************
//   T G U I T H E M E S   -   Administrate textures
//********************************************************************
Function TGUIThemes.ActivateTexture(ATextureIndex: TGUIThemesIndex): Boolean;
Begin
   Result:= False;
   With FTextures Do
      If (Count > 0) And
         (ATextureIndex >= 0) And
         (ATextureIndex < Count)
         Then Begin
            FTexture:= List[ATextureIndex];
            Result:= True;
         End;
End;
//********************************************************************
// Load TGRS_Texture
Function TGUIThemes.AddTexture(ATextureName: String;
                    ATransparentColor: TRGBAColor): TGUIThemesIndex;
Var LocalTexture: TGRS_Texture;
Begin
   Result:= -1;
   LocalTexture:= TGRS_Texture.Create;
   With LocalTexture Do Begin
      TransparentColor:= SolidBlack;
      If LoadFromFile(ATextureName,TextureDir,False)
         Then With FTextures Do Begin
            FTexture:= LocalTexture;
            Add(LocalTexture);
            Result:= Count-1;
         End
         Else LocalTexture.Free;
   End;
End;
//********************************************************************
Function TGUIThemes.TextureIndex(ATextureName: TGUITextureName): TGUIThemesIndex;
Var LocalTexture: TGRS_Texture; I: Integer;
Begin
   Result:= -1;
   ATextureName:= UpperCase(ATextureName);
   I:= 0;
   With FTextures Do If Count > 0
      Then While (I < Count) And (Result < 0) Do Begin
         LocalTexture:= List[I];
         If LocalTexture.Name <> ATextureName
            Then Inc(I) //=================>
            Else Result:= I;
      End;
End;
//********************************************************************
Procedure TGUIThemes.LoadTextures;
Var TextureData: TGUITextureData;
    TextureCount, I: Integer;
Begin
   TextureCount:= FDataProvider.LoadCount;
   For I:= 0 To TextureCount-1 Do Begin
      TextureData:= FDataProvider.LoadTextureData;
      With TextureData Do AddTexture(TextureName,TextureTranspColor);
   End;
End;
//********************************************************************
Procedure TGUIThemes.SaveTextures;
Var TextureData: TGUITextureData;
    LocalTexture: TGRS_Texture; I: Integer;
Begin
   With FTextures Do Begin
      FDataProvider.SaveCount(Count);
      For I:= 0 To Count-1 Do Begin
         LocalTexture:= List[I];
         With TextureData,LocalTexture Do begin
            TextureName:= Name;
            TextureTranspColor:= TransparentColor;
         End;
         FDataProvider.SaveTextureData(TextureData);
      End;
   End;
End;
//********************************************************************
//   T G U I T H E M E S   -   Administrate Fonts
//********************************************************************
Function TGUIThemes.ActivateFont(AFontIndex: TGUIThemesIndex): Boolean;
Begin
   Result:= False;
   With FFonts Do
      If (Count > 0) And
         (AFontIndex >= 0) And
         (AFontIndex < Count)
         Then Begin
            FFont:= List[AFontIndex];
            Result:= True;
         End;
End;
//********************************************************************
// Load TGRS_Font
Function TGUIThemes.AddFont(AFontName: String;
                 ASize,AStyle: Integer): TGUIThemesIndex;
Var LocalFont: TGRS_TTFont;
Begin
   Result:= -1;
   LocalFont:= TGRS_TTFont.Create(AFontName,FontDir);
   With LocalFont Do Begin
      Size:= ASize;
      Style:= AStyle;
      If BuildFont Then With FFonts Do Begin
         FFont:= LocalFont;
         Add(LocalFont);
         Result:= Count-1;
      End;
   End;
End;
//********************************************************************
Function TGUIThemes.FontIndex(AFontName: TGUIFontName): TGUIThemesIndex;
Var LocalFont: TGRS_TTFont; I: Integer;
Begin
   Result:= -1;
   AFontName:= UpperCase(AFontName);
   I:= 0;
   With FFonts Do If Count > 0
      Then While (I < Count) And (Result < 0) Do Begin
         LocalFont:= List[I];
         If LocalFont.Name <> AFontName
            Then Inc(I) //=================>
            Else Result:= I;
      End;
End;
//********************************************************************
Procedure TGUIThemes.LoadFonts;
Var FontData: TGUIFontData; FontCount, I: Integer;
Begin
   FontCount:= FDataProvider.LoadCount;
   For I:= 0 To FontCount-1 Do Begin
      FontData:= FDataProvider.LoadFontData;
      With FontData Do AddFont(FontName,FontSize,FontStyle);
   End;
End;
//********************************************************************
Procedure TGUIThemes.SaveFonts;
Var FontData: TGUIFontData; LocalFont: TGRS_TTFont;
    FontCount, I: Integer;
Begin
   With FFonts Do Begin
      FDataProvider.SaveCount(Count);
      For I:= 0 To Count-1 Do Begin
         LocalFont:= List[I];
         With FontData, LocalFont Do Begin
            FontName:= Name;
            FontSize:= Size;
            FontStyle:= Style;
         End;
         FDataProvider.SaveFontData(FontData);
      End;
   End;
End;
//********************************************************************
//********************************************************************
//   T G U I T H E M E S   -   Draw (ALeft,ATop,ARight,ABottom)
//********************************************************************
// Draws an ellipse
Procedure TGUIThemes.Ellipse(ALeft,ATop,ARight,ABottom: Integer;
                                             Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Ellipse2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
// Draws a 2D-line from startpoint to endpoint
Procedure TGUIThemes.Line(FromX,FromY,ToX,ToY: Integer);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Line2D(FromX,FromY,ToX,ToY);
End;
//********************************************************************
// Draws a rectangle
Procedure TGUIThemes.Rectangle(ALeft,ATop,ARight,ABottom: Integer;
                                          Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Rectangle2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
// Draws a rectangle
Procedure TGUIThemes.TexRectangle(ALeft,ATop,ARight,ABottom: Integer;
                                      WithTexture: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_TexRectangle2D(ALeft,ATop,ARight,ABottom,FTexture.Texture);
End;
//********************************************************************
// Draws a rectangle with rounded corners
Procedure TGUIThemes.RoundRect(ALeft,ATop,ARight,ABottom: Integer;
                                  Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_RoundRect2D(ALeft,ATop,ARight,ABottom,Fill);
End;
//********************************************************************
//   T G U I T H E M E S  -   Draw (Array Of TPoint2D)
//********************************************************************
// Graph plotter
Procedure TGUIThemes.Graph(APoint: Array Of TPoint2D);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Graph2D(APoint);
End;
//********************************************************************
// Draws a polygon either with lines only or filled with current color
Procedure TGUIThemes.Polygon(APoint: Array Of TPoint2D;
                                 Fill: Boolean);
Begin
   GRS_SetLineWidth(FLineWidth);
   GRS_SetColor(FColor.Color);
   GRS_Polygon2D(APoint,Fill);
End;
//********************************************************************
//********************************************************************
//   T M O U S E
//********************************************************************
Constructor TMouse.Create;
Begin
   Inherited Create;
   FMouseImage:= miArrow;
End;
//********************************************************************
Destructor TMouse.Destroy;
Begin
   Inherited Destroy;
End;
//********************************************************************
Procedure TMouse.Draw;
Begin
End;
//********************************************************************
Function TMouse.GetMouseImage: TMouseImage;
Begin Result:= FMouseImage; End;
//********************************************************************
Procedure TMouse.SetMouseImage(AMouseImage: TMouseImage);
Begin FMouseImage:= AMouseImage; End;
//********************************************************************
//********************************************************************
Begin
   MainDir:= GetCurrentDir;
   TextureDir:= MainDir + '\Textures\';
   FontDir:= MainDir + '\Fonts\';
End.
