//********************************************************************
UNIT GUIDescendants;
//********************************************************************
INTERFACE

USES GUIItem, GUIBase, GraphicSupport;
//********************************************************************
TYPE
   TShapeType = (stLine, stRectangle, stRoundRect, stEllipse);
//********************************************************************
   TGUIShape = Class(TGUIItem)
   Private
      FDown: Boolean;
      FShapeType: TShapeType;
   Public
      Constructor Create(AName: String; AParent: TGUIItem;
                      AShapeType: TShapeType); Reintroduce;

      Procedure OnMouseBtnLDown
         (AMouseBtnEventData: TMouseBtnEventData); Override;
      Procedure OnMouseBtnLUp
         (AMouseBtnEventData: TMouseBtnEventData); Override;

      Procedure OnDraw; Override;
   End;
//********************************************************************
   TGUIShiftScale = Class(TGUIItem)
   Private
      DragMode: Byte;
      FTextureIndex,FFontIndex: TGUIThemesIndex;
      StartX,StartY: Integer;
   Public
      Constructor Create(AName: String; AParent: TGUIItem); Override;

      Procedure OnMouseBtnLDown
         (AMouseBtnEventData: TMouseBtnEventData); Override;
      Procedure OnMouseBtnLUp
         (AMouseBtnEventData: TMouseBtnEventData); Override;
      Procedure OnMouseMove
         (AMouseMoveEventData: TMouseMoveEventData); Override;
      Procedure OnMouseWheelDown
         (AMouseWheelEventData: TMouseWheelEventData); Override;
      Procedure OnMouseWheelUp
         (AMouseWheelEventData: TMouseWheelEventData); Override;

      Procedure OnDraw; Override;
   End;
//********************************************************************
IMPLEMENTATION
//********************************************************************
//********************************************************************
Constructor TGUIShape.Create(AName: String; AParent: TGUIItem;
                         AShapeType: TShapeType);
Begin
   Inherited Create(AName,AParent);
   FShapeType:= AShapeType;
   FDown:= False;
End;
//********************************************************************
Procedure TGUIShape.OnMouseBtnLDown
                  (AMouseBtnEventData: TMouseBtnEventData);
Begin
   FDown:= True;
End;
//********************************************************************
Procedure TGUIShape.OnMouseBtnLUp
                  (AMouseBtnEventData: TMouseBtnEventData);
Begin
   FDown:= False;
End;
//********************************************************************
Procedure TGUIShape.OnDraw;
Begin
   Inherited OnDraw;

   With Themes Do If ActivateColor(FGColorIndex)
      Then Begin
         If FDown
            Then Themes.Color.Luminance:= 0.5;
         Case FShapeType Of
            stLine: Line(Left,Top,Right,Bottom);
            stRectangle: Rectangle(Left,Top,Right,Bottom, True);
            stRoundRect: RoundRect(Left,Top,Right,Bottom, True);
            stEllipse: Ellipse(Left,Top,Right,Bottom, True);
         End;
         If FDown
            Then Themes.Color.Reset;
      End;
End;
//********************************************************************
//********************************************************************
Constructor TGUIShiftScale.Create(AName: String; AParent: TGUIItem);
Begin
   Inherited Create(AName,AParent);
   FGColorName:= 'SolidWhite';
   With Themes Do Begin
      FTextureIndex:= TextureIndex('Default64X64');
      //FFontIndex:= FontIndex('FreeMono');
      //FFontIndex:= AddFont('SteelOut',200,
      //                TTF_STYLE_ITALIC Or TTF_STYLE_UNDERLINE);
      Font.Color:= SolidGreen;
   End;
End;
//********************************************************************
Procedure TGUIShiftScale.OnDraw;
Begin
   Inherited OnDraw;  // no BackGroundRectangle

   With Themes Do
      If ((ActivateColor(FGColorIndex)) And
          (ActivateTexture(FTextureIndex)) And
          (ActivateFont(FFontIndex)))
      Then Begin
         If DragMode <> 0
            Then Themes.Color.Luminance:= 0.5;

         //RoundRect(Left,Top,Right,Bottom, True);
         TexRectangle(Left,Top,Right,Bottom, True);

         Font.TextOut(Left+10,Top+10,'ABCDEFGHIJKLMNOPQRSTUVWXYZ');
         //Font.TextOut(Left+10,Top+10,'abcdefghijklmnopqrstuvwxyz');

         If DragMode <> 0
            Then Themes.Color.Reset;
      End;
End;
//********************************************************************
Procedure TGUIShiftScale.OnMouseBtnLDown
               (AMouseBtnEventData: TMouseBtnEventData);
Begin
   With AMouseBtnEventData Do Begin
      If EdgeZoneX(X) Then DragMode:= 2;    // Skalieren X
      If EdgeZoneY(Y) Then DragMode:= 3;    // Skalieren Y
      If EdgeZoneXY(X,Y) Then DragMode:= 4; // Skalieren X/Y
      If DragMode = 0 Then DragMode:= 1;    // Shift

      StartX:=X; StartY:=Y;
   End;
End;
//********************************************************************
Procedure TGUIShiftScale.OnMouseBtnLUp
               (AMouseBtnEventData: TMouseBtnEventData);
Begin
   DragMode:= 0;
End;
//********************************************************************
Procedure TGUIShiftScale.OnMouseMove
               (AMouseMoveEventData: TMouseMoveEventData);
Begin
   With AMouseMoveEventData Do Begin
      Case DragMode Of
         1: Begin
               Shift2D(X-StartX,Y-StartY);
               StartX:=X; StartY:=Y;
            End;
         2: ScaleX_2D(X);
         3: ScaleY_2D(Y);
         4: ScaleXY_2D(X,Y);
      End;
   End;
End;
//********************************************************************
Procedure TGUIShiftScale.OnMouseWheelDown
               (AMouseWheelEventData: TMouseWheelEventData);
Begin
   Right:= Right+1; Bottom:= Bottom+1;
End;
//********************************************************************
Procedure TGUIShiftScale.OnMouseWheelUp
            (AMouseWheelEventData: TMouseWheelEventData);
Begin
   Right:= Right-1; Bottom:= Bottom-1;
End;
//********************************************************************
//********************************************************************
End.
